/*
 * JBoss, Home of Professional Open Source
 * Copyright 2012, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.arquillian.warp.client.filter.http;

import java.net.URL;
import java.util.List;
import java.util.Map;

import org.jboss.arquillian.warp.client.filter.Request;

/**
 * An HTTP request.
 * <p>
 * <h3>Accessing Query Parameters and Cookie</h3>
 * <p>
 * Unlike the Servlet API, a query string is constructed and decomposed by
 * {@link QueryStringEncoder} and {@link QueryStringDecoder}.  {@link Cookie}
 * support is also provided separately via {@link CookieEncoder} and
 * {@link CookieDecoder}.
 *
 * @author <a href="http://www.jboss.org/netty/">The Netty Project</a>
 * @author Andy Taylor (andy.taylor@jboss.org)
 * @author Lukas Fryc
 */
public interface HttpRequest extends HttpMessage, Request {

    /**
     * Returns the method of this request.
     */
    HttpMethod getMethod();

    /**
     * Returns the URI (or path) of this request.
     */
    String getUri();

    /**
     * Returns the URL of this request
     */
    URL getUrl();

    /**
     * Returns a map of query parameters
     */
    Map<String, List<String>> getQueryParameters();

    /**
     * Returns a map of HTTP form data attributes
     */
    Map<String, List<String>> getHttpDataAttributes();
}
